package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelMapper;
import com.syc.slm.slmbi.dto.QueryInstrumentPanelDTO;
import com.syc.slm.slmbi.dto.SaveInstrumentPanelDTO;
import com.syc.slm.slmbi.entity.*;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.enums.NodeType;
import com.syc.slm.slmbi.enums.PublishedStatus;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.*;
import com.syc.slm.slmbi.vo.InstrumentPanelConfigVo;
import com.syc.slm.slmbi.vo.InstrumentPanelVo;
import com.syc.slm.slmbi.vo.PanelTreeNodeVo;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * slm-bi
 * 仪表板
 *
 * @author : lin
 * @date : 2021-03-25 16:17
 **/
@Slf4j
@Service
public class InstrumentPanelServiceImpl extends ServiceImpl<InstrumentPanelMapper, InstrumentPanel>
    implements InstrumentPanelService {
    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private InstrumentPanelPermissionService permissionService;

    @Autowired
    private InstrumentPanelConfigService configService;

    @Autowired
    private InstrumentPanelDraftService draftService;

    @Autowired
    private InstrumentPanelGroupService panelGroupService;

    @Override
    @Transactional(readOnly = true)
    public IPage<InstrumentPanelVo> selectInstrumentPanelList(String appId, QueryInstrumentPanelDTO query) {
        IPage<InstrumentPanelVo> page = new Page<>(query.getCurrent(), query.getSize());
        QueryWrapper<InstrumentPanel> where = new QueryWrapper<>();
        if(StringUtils.isEmpty(appId)){
            throw new SysException("appId必须传入");
        }
        where.eq("p.app_id", appId);
        if(StringUtils.isEmpty(query.getGroupId())){
            throw new SysException("必须选择需要查询的仪表板分组");
        }

        InstrumentPanelGroup group = panelGroupService.getById(query.getGroupId());
        if(StringUtils.isNotEmpty(group.getParentId())){
            where.eq("p.group_id", query.getGroupId());
        }

        where.eq("p.record_status", SlmConstants.DATA_VALID);
        if (StringUtils.isNotEmpty(query.getName())) {
            where.and(x->x.like("p.name", query.getName()).or().like("p.creator_name", query.getName()).or().like("p.updater_name", query.getName()));
        }

        if (StringUtils.isNotEmpty(query.getPublished())) {
            where.eq("p.published", query.getPublished());
        }
        if (query.getUpdateTimeBegin() != null) {
            where.ge("p.update_time", query.getUpdateTimeBegin());
        }
        if (query.getUpdateTimeEnd() != null) {
            where.le("p.update_time", query.getUpdateTimeEnd());
        }
        IPage<InstrumentPanelVo> response = baseMapper.selectInstrumentPanelList(page, where);
        if(ObjectUtils.isNotEmpty(response)){
            if(CollectionUtils.isNotEmpty(response.getRecords())){
                Set<String> ids = response.getRecords()
                    .stream()
                    .filter(x -> "未发布".equals(x.getPublished()))
                    .map(BaseEntity::getId)
                    .collect(Collectors.toSet());
                if(CollectionUtils.isNotEmpty(ids)){
                    List<InstrumentPanelDraft> drafts = draftService.getByIds(ids);
                    if(CollectionUtils.isNotEmpty(drafts)){
                        Set<String> draftIds = drafts.stream().map(InstrumentPanelDraft::getPanelId).collect(Collectors.toSet());
                        response.getRecords().forEach(x->{
                            if(draftIds.contains(x.getId())){
                                x.setHasWaitPublish("有未发布版本");
                            }
                        });
                    }
                }
            }

        }
        return response;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String savePanel(String appId, SaveInstrumentPanelDTO panel) {
        log.info("保存仪表板");
        return savePanelData(appId, panel);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<String> batchPublish(List<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new SysException("请选择你要操作的数据");
        }
        List<InstrumentPanel> panels = baseMapper.selectBatchIds(ids);
        if (CollectionUtils.isNotEmpty(panels)) {
            panels.forEach(x -> {
                x.setPublished(PublishedStatus.PUBLISHED.value);
                baseMapper.updateById(x);
            });
            log.info("修改配置状态为发布状态");
            configService.publishConfigByPanelIds(ids);
            log.info("修改权限状态为发布状态");
            permissionService.publishPermissionByPanelIds(ids);
            log.info("复制一份草稿");
            draftService.saveDrafts(panels);
        }
        return ids;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String delPanel(String id) {
        if (StringUtils.isEmpty(id)) {
            throw new SysException("请选择你要操作的数据");
        }
        InstrumentPanel panel = baseMapper.selectById(id);
        if (SlmConstants.DATA_DELETED.equals(panel.getRecordStatus())) {
            throw new SysException("该数据已被删除，请刷新页面");
        }
        panel.setRecordStatus(SlmConstants.DATA_DELETED);
        baseMapper.updateById(panel);

        log.info("删除该仪表板下的配置");
        delAllSetting(id);
        log.info("删除仪表板草稿表");
        InstrumentPanelDraft draft =draftService.getByPanelId(id);
        if(ObjectUtils.isNotEmpty(draft)) {
            draft.setRecordStatus(SlmConstants.DATA_DELETED);
            draftService.updateById(draft);
        }
        return id;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveAsPanel(String appId, SaveInstrumentPanelDTO panel) {
        log.info("仪表板另存为");
        return savePanelData(appId, panel);
    }

    @Override
    @Transactional(readOnly = true)
    public Boolean showRecoveryBtn(String id) {
        return draftService.getByPanelId(id) != null;
    }

    @Override
    @Transactional(readOnly = true)
    public InstrumentPanelConfigVo recoveryPanel(String id) {
        return configService.getPublishConfigByPanelId(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String offline(String id) {
        if (StringUtils.isBlank(id)) {
            throw new SysException("请选择你要操作的数据");
        }
        InstrumentPanel panel = baseMapper.selectById(id);
        if (ObjectUtils.isEmpty(panel)) {
            throw new SysException("该数据已不存在，请刷新页面");
        }
        if (SlmConstants.DATA_DELETED.equals(panel.getRecordStatus())) {
            throw new SysException("该数据已被删除，请刷新页面");
        }
        if(PublishedStatus.WAIT_PUBLISHED.value.equals(panel.getPublished())){
            throw new SysException("该仪表板已被下线，请刷新页面");
        }
        panel.setPublished(PublishedStatus.WAIT_PUBLISHED.value);
        baseMapper.updateById(panel);
        log.info("删除草稿");
        InstrumentPanelDraft draft = draftService.getByPanelId(panel.getId());
        if(ObjectUtils.isNotEmpty(draft)){
            draft.setRecordStatus(SlmConstants.DATA_DELETED);
            draftService.updateById(draft);
        }
        log.info("更新仪表板配置为待发布状态");
        configService.offlineConfig(panel.getId(), null);

        log.info("更新仪表板权限为待发布状态");
        permissionService.offlinePermission(panel.getId());
        return id;
    }

    @Override
    public void batchUpdatePanelByIds(Set<String> panelIds, Integer recordStatus) {
        if (CollectionUtils.isNotEmpty(panelIds)) {
            baseMapper.batchUpdatePanelByIds(panelIds, recordStatus);
        }
    }

    @Override
    public void checkName(@NonNull String appId, String panelId, @NonNull String panelName) {
        LambdaQueryWrapper<InstrumentPanel> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(InstrumentPanel::getName, panelName);
        wrapper.eq(InstrumentPanel::getAppId, appId);
        wrapper.eq(InstrumentPanel::getRecordStatus, SlmConstants.DATA_VALID);
        if (StringUtils.isNotEmpty(panelId)) {
            wrapper.ne(InstrumentPanel::getId, panelId);
        }
        if (baseMapper.selectCount(wrapper) > 0) {
            throw new SysException("仪表板名字:" + panelName + "已存在!");
        }
    }

    @Override
    public List<InstrumentPanel> getByIds(Set<String> panelIds) {
        return baseMapper.selectBatchIds(panelIds);
    }

    @Override
    public List<PanelTreeNodeVo> selectPanelByIds(Set<String> finalPanelIds,String name) {
        return baseMapper.selectPanelByIds(finalPanelIds,name);
    }

    private String savePanelData(String appId, SaveInstrumentPanelDTO panel) {
        if (StringUtils.isEmpty(panel.getName())) {
            throw new SysException("仪表板名字必填");
        }
        if (StringUtils.isEmpty(panel.getGroupId())) {
            throw new SysException("仪表板分组必填");
        }
        log.info("检查仪表板名称是否重复");
        this.checkName(appId, null, panel.getName());

        log.info("保存仪表板访问权限");
        String panelId = IdWorker.get32UUID();
        String accessType = AccessType.PUBLIC.value;
        if (ObjectUtils.isNotEmpty(panel.getPermission()) && StringUtils.isNotEmpty(panel.getPermission().getAccessType())) {
            accessType = panel.getPermission().getAccessType();
            permissionService.savePanelPermission(panel.getPermission().getCustomPermissions(),panelId,panel.getPermission().getAccessType(),PublishedStatus.WAIT_PUBLISHED.value);
        }
        InstrumentPanel entity = new InstrumentPanel(panelId,panel.getName(),panel.getGroupId(),accessType,PublishedStatus.WAIT_PUBLISHED.value);

        log.info("保存仪表板配置");
        if (StringUtils.isNotEmpty(panel.getConfigDetails())) {
            configService.savePanelConfig(panelId, panel.getConfigDetails(), PublishedStatus.WAIT_PUBLISHED.value);
        }

        log.info("初始化仪表板树");
        treeService.initTree(entity.getGroupId(),NodeType.PANEL.value,entity.getId(),entity.getName());

        if (CollectionUtils.isNotEmpty(panel.getAssemblys())) {
            InstrumentPanelTree tree = treeService.getById(entity.getId());
            log.info("保存仪表板组件树");
            treeService.savePanelAssemblyTree(tree, panel.getAssemblys(), entity.getId());
        } else {
            log.info("前端没有构建组件树");
        }
        log.info("保存仪表板数据");
        baseMapper.insert(entity);
        return panelId;
    }

    private void delAllSetting(String id) {
        log.info("删除仪表板对应权限");
        permissionService.delPermissionByPanelId(id, null);

        log.info("删除仪表板对应配置");
        configService.delConfig(id, null);

        log.info("删除仪表板树");
        treeService.delTreeByPanelId(id);
    }

}
