package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.dao.InstrumentPanelTreeMapper;
import com.syc.slm.slmbi.dto.InstrumentPanelAssemblyDTO;
import com.syc.slm.slmbi.entity.*;
import com.syc.slm.slmbi.enums.NodeType;
import com.syc.slm.slmbi.enums.PublishedStatus;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.*;
import com.syc.slm.slmbi.vo.*;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.filefilter.FalseFileFilter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 仪表板树聚合表
 *
 * @author lin
 * @date 2021-04-01 10:52:44
 */
@Service
@Slf4j
public class InstrumentPanelTreeServiceImpl extends ServiceImpl<InstrumentPanelTreeMapper, InstrumentPanelTree>
    implements InstrumentPanelTreeService {

    @Autowired
    private InstrumentPanelDraftService draftService;

    @Autowired
    private InstrumentPanelGroupService groupService;
    @Autowired
    private InstrumentPanelConfigService configService;

    @Autowired
    private InstrumentPanelService panelService;
    @Override
    public void batchUpdateTreeByIds(Set<String> treeIds, Integer recordStatus) {
        if (CollectionUtils.isNotEmpty(treeIds)) {
            baseMapper.batchUpdateTreeByIds(treeIds, recordStatus);
        }
    }

    @Override
    public void initTree(String nodeId, String nodeType, String bizId, String bizName) {
        InstrumentPanelTree panelTree = baseMapper.selectById(nodeId);
        InstrumentPanelTree tree = new InstrumentPanelTree();
        tree.setId(bizId);
        tree.setNodeType(nodeType);
        tree.setFullName(panelTree.getFullName() + "." + bizName);
        tree.setFullId(panelTree.getFullId() + "." + bizId);
        tree.setName(bizName);
        baseMapper.insert(tree);
    }

    @Override
    public void delTreeByPanelId(String panelId) {
        InstrumentPanelTree tree = baseMapper.selectById(panelId);
        tree.setRecordStatus(SlmConstants.DATA_DELETED);
        baseMapper.updateById(tree);
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.likeRight(InstrumentPanelTree::getFullId, tree.getFullId() + ".");
        List<InstrumentPanelTree> list = baseMapper.selectList(where);
        if (CollectionUtils.isNotEmpty(list)) {
            list.forEach(x -> {
                x.setRecordStatus(SlmConstants.DATA_DELETED);
                baseMapper.updateById(x);
            });
        }
    }

    @Override
    public void savePanelAssemblyTree(InstrumentPanelTree tree,List<InstrumentPanelAssemblyDTO> assembly,String panelId) {
        if (CollectionUtils.isNotEmpty(assembly)) {
           assembly.forEach(x->{
               if(StringUtils.isBlank(x.getName()) || StringUtils.isBlank(x.getKey())){
                   log.info("不要传入空得组件树");
               }else {
                   InstrumentPanelTree t = new InstrumentPanelTree();
                   t.setId(IdWorker.get32UUID());
                   t.setNodeType(NodeType.ASSEMBLY.value);
                   t.setFullId(tree.getFullId() + "." + t.getId());
                   t.setFullName(tree.getFullName() + "." + x.getName());
                   t.setAssemblyKey(x.getKey());
                   t.setName(x.getName());
                   baseMapper.insert(t);
               }
           });
        }

    }

    @Override
    public void delTreeByPanelFullId(String fullId) {
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.likeRight(InstrumentPanelTree::getFullId, fullId + ".");
        List<InstrumentPanelTree> trees = baseMapper.selectList(where);
        if (CollectionUtils.isNotEmpty(trees)) {
            trees.forEach(x -> {
                x.setRecordStatus(SlmConstants.DATA_DELETED);
                baseMapper.updateById(x);
            });
        }
    }

    @Override
    public void updatePanelTreeFullName(String panelId, String panelName, String groupId) {
        InstrumentPanelTree groupTree = baseMapper.selectById(groupId);
        InstrumentPanelTree tree = baseMapper.selectById(panelId);
        tree.setFullName(groupTree.getFullName() + "." + panelName);
        tree.setFullId(groupTree.getFullId() + "." + panelId);
        tree.setName(panelName);
        baseMapper.updateById(tree);
    }

    @Override
    public List<PanelTreeNodeVo> loadTrees(String appId, String name,String uri,CurrentUser user) {
        if(StringUtils.isBlank(name)){
            return loadTrees(appId,uri,user);
        }
        List<PanelTreeNodeVo> lists = Lists.newArrayList();
        R<Map<String, String>> result = setDeptAndRole(user, uri);
        Map<String, String> detach = result.detach();
        String userId=user.getUid();
        String officeIds=detach.get("deptIds");
        String roleId=detach.get("roleIds");


        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getAppId, appId);
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelTree::getNodeType, NodeType.PANEL.value);
        if (StringUtils.isNotBlank(name)) {
            where.like(InstrumentPanelTree::getFullName, name);
        }

        List<InstrumentPanelTree> trees = baseMapper.selectList(where);
        if (CollectionUtils.isNotEmpty(trees)) {
            Set<String> panelIds = trees.stream().map(BaseEntity::getId).collect(Collectors.toSet());
            List<InstrumentPanelDraft> panels = draftService.selectDraftList(appId, panelIds, officeIds, userId, roleId);
            if (CollectionUtils.isNotEmpty(panels)) {
                Set<String> publishIds = panels.stream().map(InstrumentPanelDraft::getPanelId).collect(Collectors.toSet());
                Set<String[]> finalPanelArrays = trees.stream()
                    .filter(x -> publishIds.contains(x.getId()))
                    .map(x -> x.getFullId().split("[.]"))
                    .collect(Collectors.toSet());
                Set<String> finalPanelIds = Sets.newHashSet();
                finalPanelArrays.forEach(x-> finalPanelIds.addAll(Arrays.asList(x)));

                List<PanelTreeNodeVo> panelTreeNodeVos = panelService.selectPanelByIds(finalPanelIds,name);
                if (CollectionUtils.isNotEmpty(panelTreeNodeVos)) {
                    lists.addAll(panelTreeNodeVos);
                }
                List<PanelTreeNodeVo> groupTreeNodeVos = groupService.selectGroupByIds(finalPanelIds,null);
                if (CollectionUtils.isNotEmpty(groupTreeNodeVos)) {
                    if(CollectionUtils.isEmpty(panelTreeNodeVos) && StringUtils.isNotBlank(name)){
                        Set<String> groupIds = groupTreeNodeVos.stream().filter(x -> x.getName().contains(name)).map(PanelTreeNodeVo::getId).collect(Collectors.toSet());
                        List<InstrumentPanelTree> groupTrees = baseMapper.selectBatchIds(groupIds);
                        groupIds.clear();
                        groupTrees.forEach(x->{
                            if(x.getFullId().contains(".")){
                                groupIds.addAll(Arrays.asList(x.getFullId().split("[.]")));
                            }else{
                                groupIds.add(x.getFullId());
                            }
                        });
                        groupTreeNodeVos= groupService.selectGroupByIds(groupIds,null);
                    }
                    lists.addAll(groupTreeNodeVos);
                }
            }
        }
        return lists;
    }


    @Override
    public List<PanelTreeNodeVo> loadTrees(String appId,String uri,CurrentUser user) {
        List<PanelTreeNodeVo> lists = Lists.newArrayList();
        R<Map<String, String>> result = setDeptAndRole(user, uri);
        Map<String, String> detach = result.detach();
        String userId=user.getUid();
        String officeIds=detach.get("deptIds");
        String roleId=detach.get("roleIds");
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getAppId, appId);
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelTree::getNodeType, NodeType.PANEL.value);
        List<InstrumentPanelTree> trees = baseMapper.selectList(where);
        if (CollectionUtils.isNotEmpty(trees)) {
            Set<String> panelIds = trees.stream().map(BaseEntity::getId).collect(Collectors.toSet());
            List<InstrumentPanelDraft> panels = draftService.selectDraftList(appId,panelIds,officeIds,userId,roleId);


            if(CollectionUtils.isNotEmpty(panels)) {
                Set<String> publishIds = panels.stream().map(InstrumentPanelDraft::getPanelId).collect(Collectors.toSet());
                Set<String[]> finalPanelArrays = trees.stream()
                    .filter(x -> publishIds.contains(x.getId()))
                    .map(x -> x.getFullId().split("[.]"))
                    .collect(Collectors.toSet());
                Set<String> finalPanelIds = Sets.newHashSet();
                finalPanelArrays.forEach(x -> {
                       for (int i = 0; i < x.length; i++) {
                           if (i == 3) {
                               break;
                           }
                           finalPanelIds.add(x[i]);
                       }
                });
                // 如果当前没有发布的仪表盘，仪表树就需要显示了
                if(CollectionUtils.isEmpty(finalPanelIds)){
                    return Lists.newArrayList();
                }
                List<PanelTreeNodeVo> groupTreeNodeVos = groupService.selectGroupByIds(finalPanelIds,null);
                if (CollectionUtils.isNotEmpty(groupTreeNodeVos)) {
                    lists.addAll(groupTreeNodeVos);
                }
                List<PanelTreeNodeVo> panelTreeNodeVos = panelService.selectPanelByIds(finalPanelIds,null);
                if (CollectionUtils.isNotEmpty(panelTreeNodeVos)) {
                    lists.addAll(panelTreeNodeVos);
                } else {
                    if (CollectionUtils.isNotEmpty(groupTreeNodeVos)) {
                        String groupId = groupTreeNodeVos.stream().filter(x->StringUtils.isNotBlank(x.getParentId())).findFirst().get().getId();
                        boolean flag = true;
                        int num =0;
                        while (flag) {
                            if(num>10){
                                break;
                            }
                            num++;
                            List<PanelTreeNodeVo> childrenNodes = this.children(groupId, null,uri,user);
                            if(CollectionUtils.isEmpty(childrenNodes)){
                                flag=false;
                            }
                            long count = childrenNodes.stream()
                                .filter(x -> NodeType.PANEL.value.equals(x.getNodeType()))
                                .count();
                            if (count == 0) {
                                lists.addAll(childrenNodes);
                                groupId = childrenNodes.stream()
                                    .filter(x -> NodeType.GROUP.value.equals(x.getNodeType()))
                                    .findFirst()
                                    .get()
                                    .getId();
                            } else {
                                flag = false;
                                lists.addAll(childrenNodes);
                            }
                        }
                    }
                }
            }
        }
        return lists;
    }

    @Override
    @Transactional(readOnly = true)
    public List<PanelTreeNodeVo> children(String groupId, String name,String uri,CurrentUser user) {
        List<PanelTreeNodeVo> lists = Lists.newArrayList();
        R<Map<String, String>> result = setDeptAndRole(user, uri);
        Map<String, String> detach = result.detach();
        String userId=user.getUid();
        String officeIds=detach.get("deptIds");
        String roleId=detach.get("roleIds");

        InstrumentPanelTree tree = baseMapper.selectById(groupId);
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.likeRight(InstrumentPanelTree::getFullId, tree.getFullId() + ".");
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        if (StringUtils.isNotBlank(name)) {
            where.like(InstrumentPanelTree::getFullName, name);
        }
        where.eq(InstrumentPanelTree::getNodeType, NodeType.PANEL.value);
        List<InstrumentPanelTree> trees = baseMapper.selectList(where);
        if (CollectionUtils.isNotEmpty(trees)) {
            Set<String> panelIds = trees.stream().map(BaseEntity::getId).collect(Collectors.toSet());
            List<InstrumentPanelDraft> panels = draftService.selectDraftList(tree.getAppId(),
                                                                             panelIds,
                                                                             officeIds,
                                                                             userId,
                                                                             roleId);
            if (CollectionUtils.isNotEmpty(panels)) {
                Set<String> publishIds = panels.stream().map(InstrumentPanelDraft::getPanelId).collect(Collectors.toSet());
                Set<String> finalPanelIds = trees.stream().filter(x -> publishIds.contains(x.getId())).map(x -> {
                    String fullId = x.getFullId();
                    fullId = fullId.replace(tree.getFullId() + ".", "");
                    if (fullId.contains(".")) {
                        return fullId.substring(0, fullId.indexOf("."));
                    } else {
                        return fullId;
                    }
                }).collect(Collectors.toSet());
                if (CollectionUtils.isNotEmpty(finalPanelIds)) {
                    List<PanelTreeNodeVo> panelTreeNodeVos = panelService.selectPanelByIds(finalPanelIds, name);
                    if (CollectionUtils.isNotEmpty(panelTreeNodeVos)) {
                        lists.addAll(panelTreeNodeVos);
                        finalPanelIds.addAll(panelTreeNodeVos.stream()
                                                 .filter(x -> !groupId.equals(x.getParentId()))
                                                 .map(PanelTreeNodeVo::getParentId)
                                                 .collect(Collectors.toSet()));
                    }
                    List<PanelTreeNodeVo> groupTreeNodeVos = groupService.selectGroupByIds(finalPanelIds, name);
                    if (CollectionUtils.isNotEmpty(groupTreeNodeVos)) {
                        lists.addAll(groupTreeNodeVos);
                    }
                }
            }
        }
        return lists;
    }

    @Override
    public void updateFullNameByGroupId(String groupId, String panelId, String panelName) {
        InstrumentPanelTree group = baseMapper.selectById(groupId);
        InstrumentPanelTree panelTree = baseMapper.selectById(panelId);
        String beforeFullName = panelTree.getFullName();
        String beforeFullId = panelTree.getFullId();
        log.info("更新自身的树状结构");
        panelTree.setFullName(group.getFullName()+"."+panelName);
        panelTree.setFullId(group.getFullId()+"."+panelId);
        panelTree.setName(panelName);
        baseMapper.updateById(panelTree);


        if(ObjectUtils.isNotEmpty(group)){
            log.info("更新组件的树状结构");
            LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
            where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
            where.likeRight(InstrumentPanelTree::getFullId, beforeFullId+".");
            List<InstrumentPanelTree> trees = baseMapper.selectList(where);
            if(CollectionUtils.isNotEmpty(trees)){
                trees.forEach(x->{
                    x.setFullId(x.getFullId().replace(beforeFullId, panelTree.getFullId()));
                    String name = x.getFullName().substring(beforeFullName.length()+1);
                    x.setFullName(panelTree.getFullName()+"." + name);
                    x.setName(name);
                    baseMapper.updateById(x);
                });
            }
        }
    }

    @Override
    @Transactional(readOnly = true)
    public List<PanelTreeNodeVo> copyTreeList(String appId,String name) {
        return getCopyAndCiteTrees(appId, name, false);
    }

    @Override
    @Transactional(readOnly = true)
    public List<PanelTreeNodeVo> citeTreeList(String appId, String name) {
        return getCopyAndCiteTrees(appId, name, true);
    }

    @Override
    public Map<String, String> assembly(String appId, String panelId) {
        return configService.getAssemblyConfig(panelId);
    }

    private List<PanelTreeNodeVo> getCopyAndCiteTrees(String appId, String name,boolean showAssembly) {
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getAppId, appId);
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        if(!showAssembly){
            where.ne(InstrumentPanelTree::getNodeType,NodeType.ASSEMBLY.value);
        }
        List<InstrumentPanelTree> allGroups=baseMapper.selectList(where);
        List<InstrumentPanelTree> filteredTrees=allGroups;
        Map<String, InstrumentPanelTree> allMaps=allGroups.stream().collect(Collectors.toMap(InstrumentPanelTree::getFullId, x->x));
        Map<String, InstrumentPanelTree> filteredMaps=allMaps;
        where.like(InstrumentPanelTree::getName, name);
        if(StringUtils.isNotBlank(name)){
            filteredTrees=baseMapper.selectList(where);
            filteredMaps=filteredTrees.stream().collect(Collectors.toMap(InstrumentPanelTree::getFullId, x->x));
        }
        Stack<InstrumentPanelTree> stack=new Stack<>();
        stack.addAll(filteredTrees);
        while(!stack.isEmpty()){
            InstrumentPanelTree tree = stack.pop();
            String fullId=tree.getFullId();
            int lastDot=fullId.lastIndexOf(".");
            String pid=fullId;
            while(lastDot>0){
                pid=pid.substring(0, lastDot);
                InstrumentPanelTree pTree= Optional.ofNullable(filteredMaps.get(pid)).orElse(allMaps.get(pid));
                if(pTree!=null){
                    tree.setParentId(pid.substring(pid.lastIndexOf(".")+1));
                    filteredMaps.putIfAbsent(pid, pTree);
                    stack.add(pTree);
                    break;
                }
                lastDot=pid.lastIndexOf(".");
            }
        }
        return  CollectionUtils.isNotEmpty(filteredMaps.values())?SlmConstants.gson.fromJson(SlmConstants.gson.toJson(filteredMaps.values()), new TypeToken<List<PanelTreeNodeVo>>() {}.getType()):
            Lists.newArrayList();
    }


    private  R<Map<String,String>> setDeptAndRole(CurrentUser user,String uri){
        Map<String,String> uriVariables =Maps.newHashMap();
        uriVariables.put("appId",user.getAppId());
        return  RestTemplateUtils.get(uri, new TypeToken<R<Map<String,String>>>() {}.getType(), uriVariables);

    }
}
